'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _immutable = require('immutable');

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _Paths = require('./Paths');

var _Paths2 = _interopRequireDefault(_Paths);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var SvgSketchCanvas = function (_React$Component) {
  _inherits(SvgSketchCanvas, _React$Component);

  function SvgSketchCanvas(props) {
    _classCallCheck(this, SvgSketchCanvas);

    var _this = _possibleConstructorReturn(this, _React$Component.call(this, props));

    _this.state = {
      drawMode: true,
      isDrawing: false,
      reset: false,
      resetStore: new _immutable.List(),
      redoStore: new _immutable.List(),
      currentPaths: new _immutable.List()
    };

    _this.handlePointerDown = _this.handlePointerDown.bind(_this);
    _this.handlePointerMove = _this.handlePointerMove.bind(_this);
    _this.handlePointerUp = _this.handlePointerUp.bind(_this);
    _this.getCoordinates = _this.getCoordinates.bind(_this);

    _this.exportImage = _this.exportImage.bind(_this);
    _this.exportSvg = _this.exportSvg.bind(_this);
    _this.exportPaths = _this.exportPaths.bind(_this);
    _this.loadPaths = _this.loadPaths.bind(_this);

    _this.eraseMode = _this.eraseMode.bind(_this);
    _this.clearCanvas = _this.clearCanvas.bind(_this);
    _this.undo = _this.undo.bind(_this);
    _this.redo = _this.redo.bind(_this);

    _this.svgCanvas = null;
    return _this;
  }

  /* Add event listener to Mouse up and Touch up to
  release drawing even when point goes out of canvas */


  SvgSketchCanvas.prototype.componentDidMount = function componentDidMount() {
    document.addEventListener('pointerup', this.handlePointerUp);
  };

  SvgSketchCanvas.prototype.shouldComponentUpdate = function shouldComponentUpdate(nextProps, nextState) {
    var currentPaths = nextState.currentPaths;
    var onUpdate = this.props.onUpdate;

    // Return currentPaths to parent on update

    onUpdate(currentPaths);

    return true;
  };

  SvgSketchCanvas.prototype.componentWillUnmount = function componentWillUnmount() {
    document.removeEventListener('pointerup', this.handlePointerUp);
  };

  // Converts mouse coordinates to relative coordinate based on the absolute position of svg


  SvgSketchCanvas.prototype.getCoordinates = function getCoordinates(pointerEvent) {
    var boundingArea = this.svgCanvas.getBoundingClientRect();
    return new _immutable.Map({
      x: pointerEvent.pageX - boundingArea.left,
      y: pointerEvent.pageY - boundingArea.top
    });
  };

  /* Mouse Handlers - Mouse down, move and up */

  SvgSketchCanvas.prototype.handlePointerDown = function handlePointerDown(pointerEvent) {
    // Allow only chosen pointer type
    var allowOnlyPointerType = this.props.allowOnlyPointerType;

    if (allowOnlyPointerType !== 'all' && pointerEvent.pointerType !== allowOnlyPointerType) return;

    if (pointerEvent.pointerType === 'mouse' && pointerEvent.button !== 0) return;

    var drawMode = this.state.drawMode;

    var point = this.getCoordinates(pointerEvent);

    this.setState(function (state) {
      return {
        isDrawing: true,
        redoStore: new _immutable.List(),
        currentPaths: state.currentPaths.push(new _immutable.Map({
          drawMode: drawMode,
          paths: new _immutable.List([point])
        }))
      };
    });
  };

  SvgSketchCanvas.prototype.handlePointerMove = function handlePointerMove(pointerEvent) {
    var isDrawing = this.state.isDrawing;


    if (!isDrawing) return;

    // Allow only chosen pointer type
    var allowOnlyPointerType = this.props.allowOnlyPointerType;

    if (allowOnlyPointerType !== 'all' && pointerEvent.pointerType !== allowOnlyPointerType) return;

    var point = this.getCoordinates(pointerEvent);

    this.setState(function (state) {
      return {
        currentPaths: state.currentPaths.updateIn([state.currentPaths.size - 1], function (pathMap) {
          return pathMap.updateIn(['paths'], function (list) {
            return list.push(point);
          });
        })
      };
    });
  };

  SvgSketchCanvas.prototype.handlePointerUp = function handlePointerUp(pointerEvent) {
    if (pointerEvent.pointerType === 'mouse' && pointerEvent.button !== 0) return;

    // Allow only chosen pointer type
    var allowOnlyPointerType = this.props.allowOnlyPointerType;

    if (allowOnlyPointerType !== 'all' && pointerEvent.pointerType !== allowOnlyPointerType) return;

    this.setState({
      isDrawing: false
    });
  };

  /* Mouse Handlers ends */

  /* Canvas operations */

  SvgSketchCanvas.prototype.eraseMode = function eraseMode(erase) {
    this.setState({
      drawMode: !erase
    });
  };

  SvgSketchCanvas.prototype.clearCanvas = function clearCanvas() {
    this.setState(function (state) {
      return {
        reset: true,
        resetStore: state.currentPaths,
        currentPaths: new _immutable.List()
      };
    });
  };

  SvgSketchCanvas.prototype.undo = function undo() {
    var _state = this.state,
        reset = _state.reset,
        currentPaths = _state.currentPaths;


    if (reset) {
      this.setState(function (state) {
        return {
          reset: false,
          resetStore: new _immutable.List(),
          currentPaths: state.resetStore
        };
      });
      return;
    }

    if (currentPaths.isEmpty()) return;

    this.setState(function (state) {
      return {
        redoStore: state.redoStore.push(state.currentPaths.get(-1)),
        currentPaths: state.currentPaths.pop()
      };
    });
  };

  SvgSketchCanvas.prototype.redo = function redo() {
    var redoStore = this.state.redoStore;


    if (redoStore.isEmpty()) return;

    this.setState(function (state) {
      return {
        redoStore: state.redoStore.pop(),
        currentPaths: state.currentPaths.push(state.redoStore.get(-1))
      };
    });
  };

  /* Exporting options */

  // Creates a image from SVG and renders it on canvas, then exports the canvas as image


  SvgSketchCanvas.prototype.exportImage = function exportImage(imageType) {
    var _this2 = this;

    return new Promise(function (resolve, reject) {
      try {
        var img = document.createElement('img');
        img.src = 'data:image/svg+xml;base64,' + btoa(_this2.svgCanvas.innerHTML);

        img.onload = function () {
          var canvas = document.createElement('canvas');
          canvas.setAttribute('width', _this2.svgCanvas.offsetWidth);
          canvas.setAttribute('height', _this2.svgCanvas.offsetHeight);
          canvas.getContext('2d').drawImage(img, 0, 0);

          resolve(canvas.toDataURL('image/' + imageType));
        };
      } catch (e) {
        reject(e);
      }
    });
  };

  SvgSketchCanvas.prototype.exportSvg = function exportSvg() {
    var _this3 = this;

    return new Promise(function (resolve, reject) {
      try {
        resolve(_this3.svgCanvas.innerHTML);
      } catch (e) {
        reject(e);
      }
    });
  };

  SvgSketchCanvas.prototype.exportPaths = function exportPaths() {
    var currentPaths = this.state.currentPaths;


    return new Promise(function (resolve, reject) {
      try {
        resolve(currentPaths);
      } catch (e) {
        reject(e);
      }
    });
  };

  SvgSketchCanvas.prototype.loadPaths = function loadPaths(paths) {
    this.setState(function (prevState) {
      return {
        currentPaths: paths.concat(prevState.currentPaths)
      };
    });
  };

  /* Finally!!! Render method */

  SvgSketchCanvas.prototype.render = function render() {
    var _this4 = this;

    var _props = this.props,
        width = _props.width,
        height = _props.height,
        canvasColor = _props.canvasColor,
        background = _props.background,
        strokeColor = _props.strokeColor,
        strokeWidth = _props.strokeWidth,
        eraserWidth = _props.eraserWidth,
        style = _props.style;
    var currentPaths = this.state.currentPaths;


    return _react2.default.createElement(
      'div',
      {
        role: 'presentation',
        ref: function ref(element) {
          _this4.svgCanvas = element;
        },
        style: _extends({
          touchAction: 'none',
          width: width,
          height: height
        }, style),
        onPointerDown: this.handlePointerDown,
        onPointerMove: this.handlePointerMove,
        onPointerUp: this.handlePointerUp
      },
      _react2.default.createElement(
        'svg',
        {
          version: '1.1',
          baseProfile: 'full',
          xmlns: 'http://www.w3.org/2000/svg',
          style: {
            width: '100%',
            height: '100%',
            background: background + ' ' + canvasColor
          }
        },
        _react2.default.createElement(
          'g',
          { id: 'svgCanvasPenStrokes' },
          _react2.default.createElement(_Paths2.default, {
            strokeWidth: strokeWidth,
            eraserWidth: eraserWidth,
            paths: currentPaths,
            strokeColor: strokeColor,
            canvasColor: canvasColor
          })
        )
      )
    );
  };

  return SvgSketchCanvas;
}(_react2.default.Component);

/* Default settings */

SvgSketchCanvas.defaultProps = {
  width: '100%',
  height: '100%',
  canvasColor: 'white',
  strokeColor: 'black',
  background: '',
  strokeWidth: 4,
  eraserWidth: 8,
  allowOnlyPointerType: 'pen',
  style: {
    border: '0.0625rem solid #9c9c9c',
    borderRadius: '0.25rem'
  },
  onUpdate: function onUpdate() {}
};

/* Props validation */

SvgSketchCanvas.propTypes = process.env.NODE_ENV !== "production" ? {
  width: _propTypes2.default.string,
  height: _propTypes2.default.string,
  strokeColor: _propTypes2.default.string,
  canvasColor: _propTypes2.default.string,
  background: _propTypes2.default.string,
  strokeWidth: _propTypes2.default.number,
  eraserWidth: _propTypes2.default.number,
  allowOnlyPointerType: _propTypes2.default.string,
  style: _propTypes2.default.objectOf(_propTypes2.default.string),
  onUpdate: _propTypes2.default.func
} : {};

exports.default = SvgSketchCanvas;
module.exports = exports['default'];